@@ -330,7 +330,6 @@ class Session(AbstractRenderer):
         while self._dispatching:
             task = self._dispatcherQueue.get()
             task()
-        print("exit!!! loop")
         logger.debug("Exited dispatch loop")
 
     def _dispatcher(self):
@@ -545,16 +544,19 @@ class Session(AbstractRenderer):
 
     def __repr__(self):
         active = len(self.activeSynths())
-        return f"Session({self.name}, synths={active})"
+        return f"Session({self.name}, backend={self.engine.backend}, outdev={self.engine.outdevName}, synths={active})"
 
     def _repr_html_(self):
         assert inside_jupyter()
         from . import jupytertools
+        from . import config
         active = len(self.activeSynths())
         if active:
             jupytertools.displayButton("Stop Synths", self.unschedAll)
         name = jupytertools.htmlName(self.name)
-        return f"Session({name}, synths={active})"
+        s = f"Session({name}, backend=<code>{self.engine.backend}</code>, outdev=<code>{self.engine.outdevName}</code>, synths=<code>{active}</code>)"
+        s = jupytertools.htmlSpan(s, fontsize=config['html_repr_fontsize'])
+        return s
 
     def _deallocSynthResources(self, synthid: int | float | str) -> None:
         """
@@ -758,7 +760,7 @@ class Session(AbstractRenderer):
             return oldinstr
         self.registerInstr(instr)
         if priority:
-            self.prepareSched(name, priority, block=True)
+            self.prepareInstr(name, priority, block=True)
         return instr
 
     def registeredInstrs(self) -> dict[str, Instr]:
@@ -839,7 +841,6 @@ class Session(AbstractRenderer):
         instr = self.instrs.get(name)
         if instr is None:
             raise ValueError(f"instrument {name} not registered")
-
         self._initInstr(instr)
         instrnum = self._registerInstrAtPriority(name, priority)
         body = self.generateInstrBody(instr=instr)
@@ -880,7 +881,7 @@ class Session(AbstractRenderer):
             return None
         return registry.get(priority)
 
-    def prepareSched(self,
+    def prepareInstr(self,
                      instr: str | Instr,
                      priority: int = 1,
                      block=False
@@ -908,8 +909,7 @@ class Session(AbstractRenderer):
         assert 1 <= priority <= self.numPriorities
         needssync = False
         instrname = instr if isinstance(instr, str) else instr.name
-        rinstr = self._getReifiedInstr(instrname, priority)
-        if rinstr is None:
+        if (rinstr := self._getReifiedInstr(instrname, priority)) is None:
             rinstr = self._makeReifiedInstr(instrname, priority, block=block)
             if block:
                 self.engine.sync()
@@ -941,7 +941,7 @@ class Session(AbstractRenderer):
         """
         assert isinstance(priority, int) and 1 <= priority <= self.numPriorities
         assert instrname in self.instrs
-        rinstr, needssync = self.prepareSched(instrname, priority)
+        rinstr, needssync = self.prepareInstr(instrname, priority)
         return rinstr.instrnum
 
     def assignBus(self, kind='', value: float | None = None, persist=False
@@ -1054,7 +1054,7 @@ class Session(AbstractRenderer):
         """
         sync = False
         for event in events:
-            _, needssync = self.prepareSched(instr=event.instrname,
+            _, needssync = self.prepareInstr(instr=event.instrname,
                                              priority=event.priority,
                                              block=False)
             if needssync:
@@ -1340,7 +1340,7 @@ class Session(AbstractRenderer):
             priority: the priority, 1 to the number of priorities defined in this session (10
                 by default). Can be negative: using a priority of -1 will
                 set the priority to its maximum value.
-            whenfinished: a function of the form f(synthid) -> None
+            whenfinished: a function of the form f(synthid: int|float|str) -> None
                 if given, it will be called when this instance stops
             relative: if True, delay is relative to the current time. Otherwise delay
                 is interpreted as an absolute time from the start time of the Engine.
@@ -1349,6 +1349,8 @@ class Session(AbstractRenderer):
                 acting as mixers, filters, etc.
             unique: assign a unique instr id to the event. This results in a fractional p1 which can
                 be used to adress this specific event (turnoff, modify pfields, automate, etc)
+            name: gives the created synth a name. Named events are stored at the dict ``session.namedEvents``.
+                If an active event exists with the given name, it is stopped
             kwargs: keyword arguments are interpreted as named parameters. This is needed when
                 passing positional and named arguments at the same time
 
@@ -1409,7 +1411,7 @@ class Session(AbstractRenderer):
             raise ValueError(f"Invalid priority {priority}. For this instrument the priority "
                              f"must be between {instr.minPriority} and {self.numPriorities} (including both ends)")
 
-        rinstr, needssync = self.prepareSched(instrname, priority)
+        rinstr, needssync = self.prepareInstr(instrname, priority)
         pfields5, dynargs = instr.parseSchedArgs(args=args, kws=kwargs)  # type: ignore
         if instr.controls:
             slicenum = self._dynargsAssignSlot()
@@ -1450,10 +1452,10 @@ class Session(AbstractRenderer):
             if oldsynth := self.namedEvents.get(name):
                 if oldsynth.playStatus() != 'stopped':
                     logger.info(
-                        f"An event with name {name} and status {oldsynth.playStatus()} "
-                        "already existed. It will remain active. To prevent this, stop "
-                        "it manually by checking session.namedEvents: "
-                        "``if event := session.namedEvents.get(name): event.stop()``")
+                        f"An event ({oldsynth}) with name '{name}' and status {oldsynth.playStatus()} "
+                        "already existed. It will be stopped"
+                        )
+                    oldsynth.stop()
             self.namedEvents[name] = synth
         return synth
 
@@ -1551,8 +1553,10 @@ class Session(AbstractRenderer):
             # No need to deallocate resources here, as they will be automatically
             # released when the synth is stopped
             # self._deallocSynthResources(p1)
+        if isinstance(event, Synth):
+            dealloc(self, event.p1, delay=delay, status=event.playStatus())
 
-        if isinstance(event, float):
+        elif isinstance(event, float):
             synth = self._synths.get(event)
             if not synth:
                 logger.debug(f"Event {event} not found, cannot unschedule")
@@ -1563,15 +1567,17 @@ class Session(AbstractRenderer):
                 if int(p1) == event:
                     dealloc(self, p1, delay, status=synth.playStatus())
         elif isinstance(event, str):
-            if event not in self.instrs:
+            if synth := self.namedEvents.get(event):
+                self.unsched(synth, delay=delay)
+            elif event in self.instrs:
+                for p1, synth in self._synths.items():
+                    if synth.instrname == event:
+                        dealloc(self, p1, delay, status=synth.playStatus())
+            else:
                 logger.warning(f"No instruments with the name {event} are defined")
                 return
-
-            for p1, synth in self._synths.items():
-                if synth.instrname == event:
-                    dealloc(self, p1, delay, status=synth.playStatus())
         else:
-            dealloc(self, event.p1, delay=delay, status=event.playStatus())
+            raise ValueError(f"Invalid event {event}")
 
     def unschedAll(self, future=False) -> None:
         """
@@ -2059,12 +2065,12 @@ class Session(AbstractRenderer):
         elif isinstance(source, tableproxy.TableProxy):
             tabnum = source.tabnum
             if dur == 0:
-                dur = source.duration() / speed + fadeout
+                dur = -1 if loop else source.duration() / speed + fadeout
         elif isinstance(source, tuple) and isinstance(source[0], np.ndarray) and isinstance(source[1], int):
             table = self.makeTable(source[0], sr=source[1], unique=False, block=blockread)
             tabnum = table.tabnum
             if dur == 0:
-                dur = table.duration() / speed + fadeout
+                dur = -1 if loop else table.duration() / speed + fadeout
         else:
             raise TypeError(f"Expected table number as int, TableProxy, a path to a soundfile as str or a "
                             f"tuple (samples: np.ndarray, sr: int), got {source}")
