@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from ast import In
 import time
 from abc import ABC, abstractmethod
 from functools import cache
@@ -12,16 +13,17 @@ from .schedevent import SchedEvent
 
 from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    from typing import Sequence, Mapping
+    from typing import Sequence, Mapping, Callable
     from .instr import Instr
     from .session import Session
+    from . import interact
     import numpy as np
 
 
 __all__ = (
     'Synth',
     'SynthGroup',
-    'ui'
+    'ISynth'
 )
 
 
@@ -104,18 +106,7 @@ class ISynth(ABC):
             - :meth:`Engine.eventui`
 
         """
-        return ui(self, specs=specs)
-
-
-def ui(event, specs: dict[str, tuple[float, float]]):
-    from . import interact
-    dynparams = event.dynamicParamNames(aliases=True, aliased=False)
-    if not dynparams:
-        logger.error(f"No named parameters for {event}")
-        return
-    params = {param: event.paramValue(param) for param in sorted(dynparams)}
-    paramspecs = interact.guessParamSpecs(params, ranges=specs)
-    return interact.interactSynth(event, specs=paramspecs)
+        raise NotImplementedError
 
 
 _synthStatusIcon = {
@@ -147,6 +138,7 @@ class Synth(SchedEvent, ISynth):
             defines named controls
         uniqueId: an integer identifying this synth, if applicable
 
+
     Example
     ~~~~~~~
 
@@ -168,7 +160,7 @@ class Synth(SchedEvent, ISynth):
         synths[1].automate('ktransp', [0, 0, 10, -1])
 
     """
-    __slots__ = ('session', 'autostop', '_scheduled')
+    __slots__ = ('session', 'autostop', '_scheduled', '_setCallback')
 
     def __init__(self,
                  session: Session,
@@ -182,7 +174,8 @@ class Synth(SchedEvent, ISynth):
                  controls: Mapping[str, float] | None = {},
                  controlsSlot: int = -1,
                  uniqueId=0,
-                 name=''
+                 name='',
+                 setCallback: Callable[[Synth, str, float, float]] | None = None
                  ) -> None:
         assert controls is None or isinstance(controls, dict)
         SchedEvent.__init__(self, instrname=instr.name if instr else '', start=start, dur=dur, args=args,
@@ -208,6 +201,9 @@ class Synth(SchedEvent, ISynth):
         if name and autostop:
             logger.warning("Autostop is disabled for named synths")
 
+        self._instr: Instr | None = None
+        self._setCallback = setCallback
+
     def __del__(self):
         if self.autostop:
             self.stop()
@@ -216,6 +212,18 @@ class Synth(SchedEvent, ISynth):
     def makeGroup(synths: list[Synth]) -> SynthGroup:
         return SynthGroup(synths)
 
+    def set(self, param='', value: float = 0, delay=0, **kws) -> None:
+        proceed = True
+        if param:
+            # One callback shadows the other. If needed the synth callback
+            # can call the instr callback
+            if self._setCallback:
+                proceed = self._setCallback(self, param, value, delay)
+            elif func := self.instr._setCallback:
+                proceed = func(self, param, value, delay)
+        if proceed:
+            return super().set(param, value, delay, **kws)
+
     def wait(self, pollinterval: float = 0.02, sleepfunc=time.sleep) -> None:
         """
         Wait until this synth has stopped
@@ -234,7 +242,7 @@ class Synth(SchedEvent, ISynth):
     def body(self) -> str:
         return self.session.generateInstrBody(self.instr)
 
-    def ui(self, **specs: tuple[float, float]) -> None:
+    def ui(self, **specs: tuple[float, float] | tuple[float, float, str]) -> None:
         """
         Modify dynamic (named) arguments through an interactive user-interface
 
@@ -274,7 +282,16 @@ class Synth(SchedEvent, ISynth):
 
             - :meth:`Engine.eventui`
         """
-        return ui(event=self, specs=specs)
+        if not self.playing():
+            logger.error("ui can only be shown for playing events, synth=%s", self)
+            raise ValueError("This synth is not playing, cannot show ui")
+        from . import interact
+        paramspecs = self._makeParamSpecs(specs)
+        return interact.interactSynth(self, paramspecs)
+
+    def paramSpecs(self) -> dict[str, interact.ParamSpec]:
+        instr = self.instr
+        return instr.specs or {}
 
     def _html(self, playstatus: str = '') -> str:
         from . import _palette
@@ -289,6 +306,14 @@ class Synth(SchedEvent, ISynth):
             f'{self.instr.name}</strong>:{self.p1:.4f}',
             ]
 
+        if playstatus == 'future':
+            delay = self.start - self.session.engine._realElapsedTime[0]
+            if delay > self.session.engine.extraLatency * 1.05:
+                parts.append(f"start={self.start - self.session.engine._realElapsedTime[0]:.3f}")
+
+        durstr = f"{self.dur:.3f}".rstrip("0").rstrip(".")
+        parts.append("dur=" + durstr)
+
         if self.args and len(self.args) > 1:
             i2n = self.instr.pfieldIndexToName
             argsstrs = []
@@ -435,8 +460,8 @@ class Synth(SchedEvent, ISynth):
 
         """
         if self.playStatus() == 'stopped':
-            logger.error(f"Synth {self} has already stopped, cannot "
-                         f"set param '{param}'")
+            logger.error("Synth %s has already stopped, cannot "
+                         "set param '%s'", self, param)
         return self.session._setPfield(event=self, delay=delay, param=param, value=value)
 
     def _setTable(self, param: str, value: float, delay=0.) -> None:
@@ -789,50 +814,64 @@ class SynthGroup(BaseSchedEvent):
         out = '\n'.join(lines)
         return out
 
-    def ui(self, **specs: tuple[float, float]) -> None:
+    def paramSpecs(self) -> dict[str, interact.ParamSpec]:
+        """Returns the parameter specs, if defined
+
+        The returned dict maps a dynamic parameter name
+        to a :class``csoundengine.interact.ParamSpec``,
+        which defines the value range, start value, scale
+        type (linear, log), etc.
         """
-        Modify dynamic (named) arguments through an interactive user-interface
+        allspecs = {}
+        for synth in self.synths:
+            specs = synth.paramSpecs()
+            allspecs.update(specs)
+        return allspecs
 
-        If run inside a jupyter notebook, this method will create embedded widgets
-        to control the values of the dynamic pfields of an event. Dynamic pfields
-        are those assigned to a k-variable or declared as ``|karg|`` (see below)
+    # def ui(self, **specs: tuple[float, float]) -> None:
+    #     """
+    #     Modify dynamic (named) arguments through an interactive user-interface
 
-        Args:
-            specs: map named arg to a tuple (minvalue, maxvalue), the keyword
-                is the name of the parameter, the value is a tuple with the
-                range
+    #     If run inside a jupyter notebook, this method will create embedded widgets
+    #     to control the values of the dynamic pfields of an event. Dynamic pfields
+    #     are those assigned to a k-variable or declared as ``|karg|`` (see below)
 
-        Example
-        ~~~~~~~
+    #     Args:
+    #         specs: map named arg to a tuple (minvalue, maxvalue), the keyword
+    #             is the name of the parameter, the value is a tuple with the
+    #             range
 
-        .. code::
+    #     Example
+    #     ~~~~~~~
 
-            # Inside jupyter
-            from csoundengine import *
-            s = Engine().session()
-            s.defInstr('vco', r'''
-              |kmidinote, kampdb=-12, kcutoff=3000, kres=0.9|
-              kfreq = mtof:k(kmidinote)
-              asig = vco2:a(ampdb(kampdb), kfreq)
-              asig = moogladder2(asig, kcutoff, kres)
-              asig *= linsegr:a(0, 0.1, 1, 0.1, 0)
-              outs asig, asig
-            ''')
-            synths = [
-                s.sched('vco', kmidinote=67)
-                s.sched('vco', kmidinote=69)
-            ]
-            group = SynthGroup(synths)
+    #     .. code::
 
-            # Specify the ranges for some sliders. All named parameters
-            # are assigned a widget
-            group.ui(kampdb=(-48, 0), kres=(0, 1))
+    #         # Inside jupyter
+    #         from csoundengine import *
+    #         s = Engine().session()
+    #         s.defInstr('vco', r'''
+    #           |kmidinote, kampdb=-12, kcutoff=3000, kres=0.9|
+    #           kfreq = mtof:k(kmidinote)
+    #           asig = vco2:a(ampdb(kampdb), kfreq)
+    #           asig = moogladder2(asig, kcutoff, kres)
+    #           asig *= linsegr:a(0, 0.1, 1, 0.1, 0)
+    #           outs asig, asig
+    #         ''')
+    #         synths = [
+    #             s.sched('vco', kmidinote=67)
+    #             s.sched('vco', kmidinote=69)
+    #         ]
+    #         group = SynthGroup(synths)
 
-        .. seealso::
+    #         # Specify the ranges for some sliders. All named parameters
+    #         # are assigned a widget
+    #         group.ui(kampdb=(-48, 0), kres=(0, 1))
 
-            - :meth:`Engine.eventui`
-        """
-        ui(event=self, specs=specs)
+    #     .. seealso::
+
+    #         - :meth:`Engine.eventui`
+    #     """
+    #     ui(event=self, specs=specs)
 
     def _repr_html_(self) -> str:
         from . import jupytertools
